package cn.dg32z.lon.commands.sub;

import cn.dg32z.libs.com.github.retrooper.packetevents.manager.server.ServerVersion;
import cn.dg32z.libs.com.github.retrooper.packetevents.util.Vector3i;
import cn.dg32z.libs.com.github.retrooper.packetevents.wrapper.play.server.WrapperPlayServerBlockChange;
import cn.dg32z.libs.org.jetbrains.annotations.NotNull;
import cn.dg32z.lon.LonAntiCheat;
import cn.dg32z.lon.PluginLoader;
import cn.dg32z.lon.commands.AbstractCommand;
import cn.dg32z.lon.manager.plugin.init.HookInit;
import cn.dg32z.lon.player.PlayerData;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.World;
import org.bukkit.block.Block;
import org.bukkit.block.data.BlockData;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

/* loaded from: cn_dg32z_lon_commands_sub.jar:cn/dg32z/lon/commands/sub/Setback.class */
public final class Setback extends AbstractCommand {
    private static final List<String> MODES = Arrays.asList("sync", "normal", "nonsimulating");

    public Setback() {
        super("Setback a player", "lonac.commands.setback", false);
    }

    public void execute(@NotNull CommandSender sender, @NotNull String label, @NotNull String[] args) {
        String playerName;
        String mode;
        if (args.length == 1) {
            playerName = args[0];
            mode = PluginLoader.INSTANCE.getMessageUtils().i18nWithOutPrefix("commands.setback.default-type").toLowerCase();
        } else if (args.length == 2) {
            playerName = args[0];
            mode = args[1].toLowerCase();
            if (!MODES.contains(mode)) {
                sender.sendMessage(PluginLoader.INSTANCE.getMessageUtils().i18n("commands.setback.no-type"));
                return;
            }
        } else {
            sender.sendMessage(PluginLoader.INSTANCE.getMessageUtils().i18n("commands.setback.usage"));
            return;
        }
        String permission = "lonac.commands.setback." + mode;
        if (!sender.hasPermission(permission)) {
            sender.sendMessage(PluginLoader.INSTANCE.getMessageUtils().i18n("no-permission"));
            return;
        }
        Player target = Bukkit.getPlayer(playerName);
        if (target == null) {
            sender.sendMessage(PluginLoader.INSTANCE.getMessageUtils().i18n("not-found").replace("%player%", playerName));
            return;
        }
        if (target.hasPermission("lonac.nosetback")) {
            sender.sendMessage(PluginLoader.INSTANCE.getMessageUtils().i18n(target, "commands.setback.cannot"));
            return;
        }
        PlayerData data = PluginLoader.INSTANCE.getPlayerDataManager().getPlayer(target);
        if (data == null) {
            sender.sendMessage(PluginLoader.INSTANCE.getMessageUtils().i18n(target, "not-data-user"));
            return;
        }
        String str = mode;
        boolean z = -1;
        switch (str.hashCode()) {
            case -1039745817:
                if (str.equals("normal")) {
                    z = true;
                    break;
                }
                break;
            case 3545755:
                if (str.equals("sync")) {
                    z = false;
                    break;
                }
                break;
            case 2140763246:
                if (str.equals("nonsimulating")) {
                    z = 2;
                    break;
                }
                break;
        }
        switch (z) {
            case false:
                setBackSync(data, target, mode, sender);
                return;
            case true:
                setBackNormal(data, target, mode, sender);
                return;
            case true:
                setBackNonSimulating(data, target, mode, sender);
                return;
            default:
                sender.sendMessage(PluginLoader.INSTANCE.getMessageUtils().i18n("commands.setback.no-type"));
                return;
        }
    }

    private void setBackSync(PlayerData data, Player target, String mode, CommandSender sender) {
        data.getSetbackTeleportUtil().executeForceResync();
        refreshBlocksAroundPlayer(data, target);
        sender.sendMessage(PluginLoader.INSTANCE.getMessageUtils().i18n(target, "commands.setback.message").replace("%mode%", mode));
    }

    private void setBackNormal(PlayerData data, Player target, String mode, CommandSender sender) {
        data.getSetbackTeleportUtil().executeViolationSetback();
        sender.sendMessage(PluginLoader.INSTANCE.getMessageUtils().i18n(target, "commands.setback.message").replace("%mode%", mode));
    }

    private void setBackNonSimulating(PlayerData data, Player target, String mode, CommandSender sender) {
        data.getSetbackTeleportUtil().executeNonSimulatingSetback();
        sender.sendMessage(PluginLoader.INSTANCE.getMessageUtils().i18n(target, "commands.setback.message").replace("%mode%", mode));
    }

    private void refreshBlocksAroundPlayer(PlayerData data, Player player) {
        Location loc = player.getLocation();
        World world = loc.getWorld();
        if (world == null) {
            return;
        }
        int px = loc.getBlockX();
        int py = loc.getBlockY();
        int pz = loc.getBlockZ();
        int maxY = world.getMaxHeight() - 1;
        boolean isNewVersion = LonAntiCheat.getInstance().getPacketEventsManager().getServerVersion().isNewerThanOrEquals(ServerVersion.V_1_13);
        int minChunkX = (px - 3) >> 4;
        int maxChunkX = (px + 3) >> 4;
        int minChunkZ = (pz - 3) >> 4;
        int maxChunkZ = (pz + 3) >> 4;
        for (int cx = minChunkX; cx <= maxChunkX; cx++) {
            for (int cz = minChunkZ; cz <= maxChunkZ; cz++) {
                if (!world.isChunkLoaded(cx, cz)) {
                    world.loadChunk(cx, cz);
                }
            }
        }
        for (int x = px - 3; x <= px + 3; x++) {
            for (int z = pz - 3; z <= pz + 3; z++) {
                if (world.isChunkLoaded(x >> 4, z >> 4)) {
                    for (int y = Math.max(0, py - 3); y <= Math.min(maxY, py + 3); y++) {
                        Block block = world.getBlockAt(x, y, z);
                        Vector3i position = new Vector3i(x, y, z);
                        try {
                            WrapperPlayServerBlockChange blockChange = getWrapperPlayServerBlockChange(isNewVersion, block, position);
                            HookInit.getPacketEventsHook().sendPacket(data.getUser(), blockChange);
                        } catch (Exception e) {
                        }
                    }
                }
            }
        }
    }

    @NotNull
    private static WrapperPlayServerBlockChange getWrapperPlayServerBlockChange(boolean isNewVersion, Block block, Vector3i position) {
        WrapperPlayServerBlockChange blockChange;
        if (isNewVersion) {
            BlockData blockData = block.getBlockData();
            blockChange = new WrapperPlayServerBlockChange(position, blockData.getMaterial().getId());
        } else {
            int combined = (block.getType().getId() << 4) | (block.getData() & 15);
            blockChange = new WrapperPlayServerBlockChange(position, combined);
        }
        return blockChange;
    }

    public List<String> tabCompleter(@NotNull CommandSender sender, @NotNull String label, @NotNull String[] args) {
        if (args.length == 1) {
            String partialName = args[0].toLowerCase();
            List<String> list = new ArrayList<>();
            for (Player player : Bukkit.getOnlinePlayers()) {
                String name = player.getName();
                if (name.toLowerCase().startsWith(partialName)) {
                    list.add(name);
                }
            }
            return list;
        }
        if (args.length == 2) {
            String partial = args[1].toLowerCase();
            List<String> list2 = new ArrayList<>();
            for (String mode : MODES) {
                if (mode.startsWith(partial) && sender.hasPermission("lonac.commands.setback." + mode)) {
                    list2.add(mode);
                }
            }
            return list2;
        }
        return new ArrayList();
    }
}
